from pathlib import Path

SUPERGLUE_CWD = Path.cwd()

JOBS_PATH = SUPERGLUE_CWD / "jobs"
MODULES_PATH = SUPERGLUE_CWD / "modules"
NOTEBOOKS_PATH = SUPERGLUE_CWD / "notebooks"
TOOLS_PATH = SUPERGLUE_CWD / "tools"
TESTS_PATH = SUPERGLUE_CWD / "tests"
